#include "filterui.h"
#include "ui_filterui.h"

#include <QComboBox>
#include <QMetaProperty>
#include <QTableWidgetItem>
#include <QSpinBox>
#include <QDoubleSpinBox>
#include <QDateEdit>
#include <QDateTimeEdit>
#include <QLineEdit>
#include <QToolButton>

FilterUi::FilterUi(QWidget *parent, QObject *entity) :
    QWidget(parent),
    ui(new Ui::FilterUi)
{
    ui->setupUi(this);
    m_entity = entity;

    if (entity == NULL)
    {
        return;
    }

    m_entity->setParent(this);
    addRow(false);

    m_contextMenu = new QMenu(this);
    m_contextMenu->addAction(ui->actionAdd_condition_AND);
    m_contextMenu->addAction(ui->actionAdd_condition_OR);
}

FilterUi::~FilterUi()
{
    delete ui;
}

void FilterUi::on_actionAdd_condition_AND_triggered()
{
    addRow(false);
}

void FilterUi::on_actionAdd_condition_OR_triggered()
{
    addRow(true);
}

void FilterUi::addRow(bool isOr)
{
    if (m_entity == NULL)
    {
        Q_ASSERT(false);
        return;
    }

    int row = ui->tableWidget->rowCount();
    ui->tableWidget->setRowCount(row + 1);
    QComboBox *properties = new QComboBox(this);

    for (int i = 1; i < m_entity->metaObject()->propertyCount(); i++)
    {
        properties->addItem(tr(m_entity->metaObject()->property(i).name()), m_entity->metaObject()->property(i).name());
    }

    QComboBox *oper = new QComboBox(this);
    oper->addItems(QStringList() << "==" << "%" << "!=" << "<" << "<=" << ">" << ">=");
    ui->tableWidget->setCellWidget(row, 2, oper);

    void (QComboBox::*indexChanged)(int) = &QComboBox::currentIndexChanged;
    connect(properties, indexChanged, [this, oper, row](int index){
        this->propertyChanged(row, oper, index);
    });

    ui->tableWidget->setCellWidget(row, 1, properties);

    if (row > 0)
    {
        QComboBox *rowOper = new QComboBox(this);
        rowOper->addItems(QStringList() << tr("OR") << tr("AND"));
        rowOper->setCurrentIndex(isOr ? 0 : 1);
        ui->tableWidget->setCellWidget(row, 0, rowOper);

        QToolButton *tb = new QToolButton(this);
        tb->setText("-");
        connect(tb, &QToolButton::clicked, [this, tb](){
            int row = ui->tableWidget->rowAt(tb->y());
            ui->tableWidget->removeRow(row);
        });
        ui->tableWidget->setCellWidget(row, 4, tb);
    }
    else
    {
        ui->tableWidget->setColumnWidth(1, 150);
        ui->tableWidget->setColumnWidth(2, 50);
        ui->tableWidget->setColumnWidth(3, 300);
        ui->tableWidget->setColumnWidth(4, 50);
    }

    propertyChanged(row, oper, 0);
}

QString FilterUi::buildExpression() const
{
    QString expr;

    for (int i = 0; i < ui->tableWidget->rowCount(); i++)
    {
        if (i == 0)
        {
            expr = qobject_cast<QComboBox*>(ui->tableWidget->cellWidget(i, 1))->currentData(Qt::UserRole).toString();
        }
        else
        {
            expr += " " + QString(qobject_cast<QComboBox*>(ui->tableWidget->cellWidget(i, 0))->currentIndex() == 0 ? "||" : "&&");
            expr += " " + qobject_cast<QComboBox*>(ui->tableWidget->cellWidget(i, 1))->currentData(Qt::UserRole).toString();
        }

        expr += " " + qobject_cast<QComboBox*>(ui->tableWidget->cellWidget(i, 2))->currentText();

        QWidget *cellWidget = ui->tableWidget->cellWidget(i, 3);

        if (cellWidget != NULL)
        {
            expr += " " + cellWidget->property(cellWidget->metaObject()->userProperty().name()).toString();
        }
    }

    return expr;
}


void FilterUi::on_tableWidget_customContextMenuRequested(const QPoint &pos)
{
    m_contextMenu->popup(ui->tableWidget->viewport()->mapToGlobal(pos));
}

void FilterUi::on_actionRemove_condition_triggered()
{
    QObject *action = sender();
    ui->tableWidget->removeRow(action->property("row").toInt());
}

void FilterUi::on_btnGo_toggled(bool checked)
{
    if (checked)
    {
        emit applyFilter(buildExpression());
    }
    else
    {
        emit restoreData();
    }
}

void FilterUi::propertyChanged(int row, QComboBox *oper, int index)
{
    QWidget *cellWidget = ui->tableWidget->cellWidget(row, 3);
    ui->tableWidget->removeCellWidget(row, 3);

    if (cellWidget != NULL)
    {
        delete cellWidget;
        cellWidget = NULL;
    }

    oper->clear();

    switch (this->m_entity->metaObject()->property(index + 1).type()) {
    case QVariant::Bool:
        oper->addItems(QStringList() << "==" << "!=");
        cellWidget = new QComboBox(this);
        qobject_cast<QComboBox*>(cellWidget)->addItem("true", 1);
        qobject_cast<QComboBox*>(cellWidget)->addItem("false", 0);
        break;
    case QVariant::String:
        oper->addItems(QStringList() << "==" << "%" << "!=" << "<" << "<=" << ">" << ">=");
        cellWidget = new QLineEdit(this);
        break;
    case QVariant::Int:
        oper->addItems(QStringList() << "==" << "!=" << "<" << "<=" << ">" << ">=");
        cellWidget = new QSpinBox(this);
        break;
    case QVariant::Double:
        oper->addItems(QStringList() << "==" << "!=" << "<" << "<=" << ">" << ">=");
        cellWidget = new QDoubleSpinBox(this);
        break;
    case QVariant::Date:
        oper->addItems(QStringList() << "==" << "!=" << "<" << "<=" << ">" << ">=");
        cellWidget = new QDateEdit(this);
        qobject_cast<QDateEdit*>(cellWidget)->setCalendarPopup(true);
        break;
    case QVariant::DateTime:
        oper->addItems(QStringList() << "==" << "!=" << "<" << "<=" << ">" << ">=");
        cellWidget = new QDateTimeEdit(this);
        qobject_cast<QDateTimeEdit*>(cellWidget)->setCalendarPopup(true);
        break;
    default:
        oper->addItems(QStringList() << "==" << "!=" << "<" << "<=" << ">" << ">=");
        cellWidget = new QLineEdit(this);
        break;
    }

    if (cellWidget != NULL)
    {
        ui->tableWidget->setCellWidget(row, 3, cellWidget);
    }
}
